% Population model with corresponding equations as described in the paper 
% Only use on set of input parameters (Expt. 1 or Expt. 2) at a time.

% EC10 and EC50 were calculated manually 
% 1) Open DZn: values are final D. magna population weight, columns corresponds with Low P, Med P and High P, rows correspond with Zn concentrations raning between 1 and 1000 g Zn/L.
% 2) Calculate 10 or 50% value of 1 g Zn/L values and find the corresponding Zn concentration

clear all
close all

%% Expt. 1 input parameters
% X(1:1:1000,1)=8.4; % initial algal density (mg algae dw/L)
% D(1:1:1000,1)=0.2; % initial daphnid density (mg daphnia dw/L)
% EC50X=250; % EC50 of zinc on algae (g Zn/L) - from single species test
% EC50D=365; % EC50 of zinc on daphnia (g Zn/L) - from single species test
% bX=3.3; % Hill slope algae (-) - from single species test
% bD=7.7; % Hill slope daphnia (-) - from single species test
% mu_X_max=1.37; % (calibrated) maximum algal growth rate (1/day) - from single species test
% mu_D_max=0.3763; % (calibrated) maximum daphnid growth rate (1/day) - from single species test
% PX(1:1:1000,1)=1; % initial algal cell P quota (%)
% PD(1:1:1000,1)=1.4; % initial daphnid P quota (%)

%% Expt. 2 input parameters
X(1:1000,1)=6.3; % initial algal density (mg algae dw/L)
D(1:1000,1)=0.2; % initial daphnid density (mg daphnia dw/L)
EC50X=80; % EC50 of zinc on algae (g Zn/L) - from single species test
EC50D=365; % EC50 of zinc on daphnia (g Zn/L) - from single species test
bX=2; % Hill slope algae (-) - from single species test
bD=7.7; % Hill slope daphnia (-) - from single species test
mu_X_max=0.854; % (calibrated) maximum algal growth rate (1/day)
mu_D_max=0.3763; % (calibrated) maximum daphnid growth rate (1/day) - from single species test
PX(1:1:1000,1)=1.5; % initial algae cell P quota (%)
PD(1:1:1000,1)=1.4; % initial daphnid P quota (%)

%% Zinc and P parameters
Zn=1:1:1000; % Zn concentrations (g Zn/L)
Psol(1:1:1000,1)=0; % initial P concentration in solution (mg P/L)
Psolo(1:1:1000,1)=0; % P added in solution (mg P/l)
Prate=[0.0003875; 0.00155; 0.0062];%  P addition rate (mg P/(L*day)) for Low P, Med P and High P treatment

%% Other parameters
Palgae(1:1:1000,1)=PX(1,1)/100*X(1,1); % P in algae (mg P/L)
Pdaphnia(1:1:1000,1)=PD(1,1)/100*D(1,1); % P in daphnia (mg P/L), not used in the model
j(1:1:1000,1)=0; % initial daphnid consumption rate (mg algal dw/day)
mu_D(1:1:1000,1)=0;% initial daphnid growth rate (1/day)
mu_X(1:1:1000,1)=0;% initial algal growth rate (1/day)
Kx=250; % algal carrying capacity (mg algae/L) 
GGE_mass=0.35;% Gross growth efficiency=yield=growth/ingestion (mg daphnid dw/mg algal dw)
Pmin_X=0.2; % minimum algal P quota (%)
Popt_X=1.7; % optimal algal P quota for maximal growth rate (%)

%% For figures and calculations
D1(1:1:3,1)=D(1,1);DZn=D1;X1(1:1:3,1)=X(1,1);XZn=X1;mu_D1(1:3,1)=mu_D(1,1);mu_X1(1:3,1)=mu_X(1,1);j1(1:3,1)=j(1,1);D2=D1;X2=X1;mu_D2=mu_D1;mu_X2=mu_X1;j2=j1;D3=D1;X3=X1;mu_D3=mu_D1;mu_X3=mu_X1;j3=j1;
Psol1(1:3,1)=0;Psolo1=Psol1;PX1(1:3,1)=PX(1,1);PD1(1:3,1)=PD(1,1);Palgae1(1:3,1)=Palgae(1,1);Pdaphnia1(1:3,1)=Pdaphnia(1,1);Psol2=Psol1;Psolo2=Psol2;PX2=PX1;PD2=PD1;Palgae2=Palgae1;Pdaphnia2=Pdaphnia1;Psol3=Psol1;Psolo3=Psol3;PX3=PX1;PD3=PD1;Palgae3=Palgae1;Pdaphnia3=Pdaphnia1;

%% Model
for iP=1:1:3 % 3 P levels: Low P, Med P and High P
for iZn=1:1:1000 % 1000 Zn levels: from 1 to 1000 g Zn/L
for i=2:1:2100 % 2100 time steps, each time step is 0.01 day
       
    % Algal and daphnid growth
       mu_D(iZn,i)=(mu_D_max*PX(iZn,i-1)*X(iZn,i-1))/((PX(iZn,i-1)+0.05141)*(X(iZn,i-1)+1)*(1+(Zn(iZn)/EC50D)^bD)); % Equation 2
       mu_X(iZn,i)=mu_X_max*(PX(iZn,i-1)-Pmin_X)/((PX(iZn,i-1)-Pmin_X+0.6)*(1+(Zn(iZn)/EC50X)^bX)); % Equation 5

       D(iZn,i)=D(iZn,i-1)*exp(mu_D(iZn,i)*0.01); % Equation 1
       j(iZn,i)=mu_D(iZn,i)*D(iZn,i)/GGE_mass; % Equation 3
           if X(iZn,i-1)*exp(mu_X(iZn,i)*0.01*(1-X(iZn,i-1)/Kx))-j(iZn,i)*0.01<=0 % Boundary condition for when algal density is lower than the daphnid consumption rate at a given timestep -> algal density stays 0 or positive.
               j(iZn,i)=X(iZn,i-1)*exp(mu_X(iZn,i)*0.01*(1-X(iZn,i-1)/Kx))/0.01;
           end
       X(iZn,i)=X(iZn,i-1)*exp(mu_X(iZn,i)*0.01*(1-X(iZn,i-1)/Kx))-j(iZn,i)*0.01; % Equation 4

     % P in solution, in algae and in daphnia.
       k=mu_X_max*Popt_X/100; % Equation 6
       
       if rem(i/100,1)==0 % P is only added in a single dose once per day
           if Psol(iZn,i-1)-k*0.01*X(iZn,i)+Prate(iP)*i/100<0 % Boundary condition for when solution P is lower than algal P uptake rate at a given time point -> P in solution stays 0 or positive
               k=(Psol(iZn,i-1)+Prate(iP)*i/100)/(0.01*X(iZn,i));
           end
           Psol(iZn,i)=Psol(iZn,i-1)-k*0.01*X(iZn,i)+Prate(iP)*i/100; % Equation 9
           Psolo(iZn,i)=Psolo(iZn,i-1)+Prate(iP)*i/100;
       else
           if Psol(iZn,i-1)-k*0.01*X(iZn,i)<0 % Boundary condition for when solution P is lower than algal P uptake rate at a given time point -> P in solution stays 0 or positive
               k=Psol(iZn,i-1)/(0.01*X(iZn,i));
           end
           Psol(iZn,i)=Psol(iZn,i-1)-k*0.01*X(iZn,i); % Equation 9
           Psolo(iZn,i)=Psolo(iZn,i-1);
       end
       
       Pdaphnia(iZn,i)=Pdaphnia(iZn,i-1)+j(iZn,i)/10000*PX(iZn,i-1); %not used in the model
       Palgae(iZn,i)=Palgae(iZn,i-1)+k*0.01*X(iZn,i)-j(iZn,i)*PX(iZn,i-1)/100*0.01; % Equation 7
       
       if  X(iZn,i)>0
           PX(iZn,i)=Palgae(iZn,i)/X(iZn,i)*100; % Equation 8
       else
           PX(iZn,i)=0; 
       end

      % code for figures
       if iZn==100
           X1(iP,i)=X(iZn,i);D1(iP,i)=D(iZn,i);mu_D1(iP,i)=mu_D(iZn,i);mu_X1(iP,i)=mu_X(iZn,i);j1(iP,i)=j(iZn,i);
           PX1(iP,i)=PX(iZn,i);Palgae1(iP,i)=Palgae(iZn,i);Pdaphnia1(iP,i)=Pdaphnia(iZn,i);Psol1(iP,i)=Psol(iZn,i);Psolo1(iP,i)=Psolo(iZn,i); 
       end
       if iZn==EC50X
           X2(iP,i)=X(iZn,i);D2(iP,i)=D(iZn,i);mu_D2(iP,i)=mu_D(iZn,i);mu_X2(iP,i)=mu_X(iZn,i);j2(iP,i)=j(iZn,i);
           PX2(iP,i)=PX(iZn,i);Palgae2(iP,i)=Palgae(iZn,i);Pdaphnia2(iP,i)=Pdaphnia(iZn,i);Psol2(iP,i)=Psol(iZn,i);Psolo2(iP,i)=Psolo(iZn,i); 
       end
       if iZn==EC50D
           X3(iP,i)=X(iZn,i);D3(iP,i)=D(iZn,i);mu_D3(iP,i)=mu_D(iZn,i);mu_X3(iP,i)=mu_X(iZn,i);j3(iP,i)=j(iZn,i);
           PX3(iP,i)=PX(iZn,i);Palgae3(iP,i)=Palgae(iZn,i);Pdaphnia3(iP,i)=Pdaphnia(iZn,i);Psol3(iP,i)=Psol(iZn,i);Psolo3(iP,i)=Psolo(iZn,i); 
       end
        
       if i==2100
           DZn(iZn,iP)=D(iZn,i);
           XZn(iZn,iP)=X(iZn,i);
       end
end
end
end

%% Total P mass balance
Ptotal1=Palgae1+Pdaphnia1+Psol1-Psolo1; % Equation 10 - Low P
Ptotal2=Palgae2+Pdaphnia2+Psol2-Psolo2; % Equation 10 - Med P
Ptotal3=Palgae3+Pdaphnia3+Psol3-Psolo3; % Equation 10 - High P

%% Output graphs
% graph settings
co = [0 0.4470 0.7410;0.8500 0.3250 0.0980;0.9290 0.6940 0.1250;0 0.4470 0.7410;0.8500 0.3250 0.0980;0.9290 0.6940 0.1250; 0 0 0];
set(groot,'defaultAxesColorOrder',co)
t=1:1:2100;% time (1/100 day)

figure % Modelled dose-response curve at 3 P levels after 21 days
semilogx(Zn,DZn), legend('Low P','Med P','High P'), title('Modelled for microcosm experiment'), xlabel('Zn concentration (g Zn/l)'), ylabel('Daphnid population weight (mg dw/L)'), xlim([4,1000]) 

figure %  Algal and daphnid population dynamics in time at 3 Zn levels
subplot(3,1,1), plot(t/100,X1), hold on, plot(t/100,D1,'--'), xlim([0,21]), title({'Algal (full line) and daphnid (dashed line) population dynamics in time';'at 0 g Zn/L (top), EC50 of algae (middle) and EC50 of daphnia (bottom)'}),legend('Low P','Med P','High P','Location','northwest')
subplot(3,1,2), plot(t/100,X2), hold on, plot(t/100,D2,'--'), xlim([0,21]), ylabel('Daphnid/algal population weight (mg dw/L)')
subplot(3,1,3), plot(t/100,X3), hold on, plot(t/100,D3,'--'), xlim([0,21]), xlabel('Time (days)')

figure % Plots with intermediate model information
subplot(3,4,1), plot(t/100,Psol1), xlim([0,21]), ylim([0,0.4]), title({'P in solution';'(mg P/L)'}), ylabel('At 0 g Zn/L')
subplot(3,4,2), plot(t/100,PX1), xlim([0,21]), ylim([0,2.2]), title({'algal P quota';'(%P)'})
subplot(3,4,3), plot(t/100,mu_X1), xlim([0,21]), title({'algal growth rate';'(1/day)'})
subplot(3,4,4), plot(t/100,mu_D1), xlim([0,21]), title({'daphnid growth rate';'(1/day)'})
subplot(3,4,5), plot(t/100,Psol2), xlim([0,21]), ylim([0,0.4]), ylabel('At algal EC50')
subplot(3,4,6), plot(t/100,PX2), xlim([0,21]), ylim([0,2.2])
subplot(3,4,7), plot(t/100,mu_X2), xlim([0,21])
subplot(3,4,8), plot(t/100,mu_D2), xlim([0,21])
subplot(3,4,9), plot(t/100,Psol3), xlabel('Time (days)'), xlim([0,21]), ylim([0,0.4]), ylabel('At daphnid EC50')
subplot(3,4,10), plot(t/100,PX3), xlabel('Time (days)'), xlim([0,21]), ylim([0,2.2])
subplot(3,4,11), plot(t/100,mu_X3), xlabel('Time (days)'), xlim([0,21])
subplot(3,4,12), plot(t/100,mu_D3), xlabel('Time (days)'), xlim([0,21])

